==> fhcore-libs-0.2.5.4.tar.gz: fhcore-libs-0.2.5.4/fhcore/forms.py <==
# -*- coding: utf-8 -*-
""" ENTRIES MODULE > FORMS
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""
from django import forms
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.db.models.aggregates import Max
from django.utils.translation import get_language, ugettext as _
from tinymce.widgets import TinyMCE

from fhcore.models.mixins import LANG_CHOICES

from .widgets import AutocompleteInput, AutocompleteWithFilterInput

class LocaleModelMixinForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        if not kwargs.has_key('instance') and self.base_fields.has_key('lang'):
            #self.base_fields['lang'].initial = get_language()[:2] or settings.LANGUAGES[0][0]
            self.base_fields['lang'].required = True
        super(LocaleModelMixinForm, self).__init__(*args, **kwargs)

class OrderableModelMixinForm(forms.ModelForm):
    
    order_field = "order"
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key(self.order_field):
            if not kwargs.has_key('instance'):
                # metti in prima posizione
                # self.base_fields[self.order_field].initial = 0
                # metti in ultima posizione
                order__max = self._meta.model.objects.all().aggregate(Max(self.order_field))['order__max'] 
                self.base_fields[self.order_field].initial = order__max and order__max+1 or 0
        super(OrderableModelMixinForm, self).__init__(*args, **kwargs)

class OrderableFirstModelMixinForm(forms.ModelForm):
    
    order_field = "order"
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key(self.order_field):
            if not kwargs.has_key('instance'):
                # metti in prima posizione
                # self.base_fields[self.order_field].initial = 0
                # metti in ultima posizione
                order__max = self._meta.model.objects.all().aggregate(Max(self.order_field))['order__max'] 
                self.base_fields[self.order_field].initial = order__max and order__max+1 or 0
        super(OrderableFirstModelMixinForm, self).__init__(*args, **kwargs)

class BaseEntryModelForm(forms.ModelForm):
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key('excerpt'):
            self.base_fields['excerpt'].widget = TinyMCE(mce_attrs=settings.TINYMCE_BASE_CONFIG)
        if self.base_fields.has_key('content'):
            self.base_fields['content'].widget = TinyMCE(mce_attrs=settings.TINYMCE_ADVANCED_CONFIG)
        super(BaseEntryModelForm, self).__init__(*args, **kwargs)

class AdvancedLocaleModelForm(OrderableModelMixinForm, LocaleModelMixinForm, BaseEntryModelForm):

    def __init__(self, *args, **kwargs):
        """
        if self.base_fields.has_key('sites'):
            current_site = Site.objects.filter(id=settings.SITE_ID) 
            self.base_fields['sites'].initial = current_site
        """
        super(AdvancedLocaleModelForm, self).__init__(*args, **kwargs)

class AdvancedEntryModelForm(LocaleModelMixinForm, BaseEntryModelForm):
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key('object_id') and self.base_fields.has_key('content_type'):
            try:
                self.base_fields['object_id'].widget = AutocompleteWithFilterInput(view_url=reverse('autocomplete-ctype-view'), c_type_id='id_content_type')    
            except Exception, ex:
                pass
        if self.base_fields.has_key('tags'):
            try:
                self.base_fields['tags'].widget = AutocompleteInput(view_url=reverse('autocomplete-tag-view'))    
            except Exception, ex:
                pass
        if self.base_fields.has_key('sites'):
            current_site = Site.objects.filter(id=settings.SITE_ID) 
            self.base_fields['sites'].initial = current_site
            self.base_fields['sites'].required = False
        super(AdvancedEntryModelForm, self).__init__(*args, **kwargs)
        
    def clean_sites(self, *args, **kwargs):
        if not self.cleaned_data['sites']:
            current_site = Site.objects.filter(id=settings.SITE_ID) 
            self.cleaned_data['sites'] = current_site
        return self.cleaned_data['sites']  
    
    
    
def get_form_class_for(obj, includes=[], excludes=[]):
    """
    Factory for creating Forms without using modelformfactory
    the model has to overrive SelfAwareMixin class
    Taken from http://www.slideshare.net/erics1/powerful-generic-patterns-with-django
    Use :
    > obj = get_object(content_type_id, object_id)
    > formclass = get_form_class_for(obj)
    > form = formclass()
    """
    
    modelclass = obj.get_ct().model_class()
    
    class _MagicForm(forms.ModelForm):
        
        class Meta:
            model= modelclass
            if includes:
                fields = includes
            if excludes:
                exclude = excludes
                
    return _MagicForm

==> fhcore-libs-0.2.6.2.tar.gz: fhcore-libs-0.2.6.2/fhcore/forms.py <==
# -*- coding: utf-8 -*-
""" ENTRIES MODULE > FORMS
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""
import logging
from django import forms
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.db.models.aggregates import Max
from django.forms.models import BaseInlineFormSet
from django.utils.translation import get_language, ugettext as _

from fhcore.models.mixins import LANG_CHOICES

from .widgets import AutocompleteInput, AutocompleteWithFilterInput

logger = logging.getLogger(__name__)

try:
    from tinymce.widgets import TinyMCE
    TINYMCE_IMPORTED = True
except ImportError as ex:
    TINYMCE_IMPORTED = False
    logger.info('TinyMCE not imported')





class LocaleModelMixinForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        if not kwargs.has_key('instance') and self.base_fields.has_key('lang'):
            #self.base_fields['lang'].initial = get_language()[:2] or settings.LANGUAGES[0][0]
            self.base_fields['lang'].required = True
        super(LocaleModelMixinForm, self).__init__(*args, **kwargs)

class OrderableModelMixinForm(forms.ModelForm):
    
    order_field = "order"
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key(self.order_field):
            if not kwargs.has_key('instance'):
                # metti in prima posizione
                # self.base_fields[self.order_field].initial = 0
                # metti in ultima posizione
                order__max = self._meta.model.objects.all().aggregate(Max(self.order_field))['order__max'] 
                self.base_fields[self.order_field].initial = order__max and order__max+1 or 0
        super(OrderableModelMixinForm, self).__init__(*args, **kwargs)

class OrderableFirstModelMixinForm(forms.ModelForm):
    
    order_field = "order"
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key(self.order_field):
            if not kwargs.has_key('instance'):
                # metti in prima posizione
                # self.base_fields[self.order_field].initial = 0
                # metti in ultima posizione
                order__max = self._meta.model.objects.all().aggregate(Max(self.order_field))['order__max'] 
                self.base_fields[self.order_field].initial = order__max and order__max+1 or 0
        super(OrderableFirstModelMixinForm, self).__init__(*args, **kwargs)

class BaseEntryModelForm(forms.ModelForm):
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key('excerpt') and TINYMCE_IMPORTED:
            self.base_fields['excerpt'].widget = TinyMCE(mce_attrs=settings.TINYMCE_BASE_CONFIG)
        if self.base_fields.has_key('content') and TINYMCE_IMPORTED:
            self.base_fields['content'].widget = TinyMCE(mce_attrs=settings.TINYMCE_ADVANCED_CONFIG)
        super(BaseEntryModelForm, self).__init__(*args, **kwargs)

class AdvancedLocaleModelForm(OrderableModelMixinForm, LocaleModelMixinForm, BaseEntryModelForm):

    def __init__(self, *args, **kwargs):
        """
        if self.base_fields.has_key('sites'):
            current_site = Site.objects.filter(id=settings.SITE_ID) 
            self.base_fields['sites'].initial = current_site
        """
        super(AdvancedLocaleModelForm, self).__init__(*args, **kwargs)

class AdvancedEntryModelForm(LocaleModelMixinForm, BaseEntryModelForm):
    
    def __init__(self, *args, **kwargs):
        if self.base_fields.has_key('object_id') and self.base_fields.has_key('content_type'):
            try:
                self.base_fields['object_id'].widget = AutocompleteWithFilterInput(view_url=reverse('autocomplete-ctype-view'), c_type_id='id_content_type')    
            except Exception, ex:
                pass
        if self.base_fields.has_key('tags'):
            try:
                self.base_fields['tags'].widget = AutocompleteInput(view_url=reverse('autocomplete-tag-view'))    
            except Exception, ex:
                pass
        if self.base_fields.has_key('sites'):
            current_site = Site.objects.filter(id=settings.SITE_ID) 
            self.base_fields['sites'].initial = current_site
            self.base_fields['sites'].required = False
        super(AdvancedEntryModelForm, self).__init__(*args, **kwargs)
        
    def clean_sites(self, *args, **kwargs):
        if not self.cleaned_data['sites']:
            current_site = Site.objects.filter(id=settings.SITE_ID) 
            self.cleaned_data['sites'] = current_site
        return self.cleaned_data['sites']  
    
    
    
def get_form_class_for(obj, includes=[], excludes=[]):
    """
    Factory for creating Forms without using modelformfactory
    the model has to overrive SelfAwareMixin class
    Taken from http://www.slideshare.net/erics1/powerful-generic-patterns-with-django
    Use :
    > obj = get_object(content_type_id, object_id)
    > formclass = get_form_class_for(obj)
    > form = formclass()
    """
    
    modelclass = obj.get_ct().model_class()
    
    class _MagicForm(forms.ModelForm):
        
        class Meta:
            model= modelclass
            if includes:
                fields = includes
            if excludes:
                exclude = excludes
                
    return _MagicForm


class RequireOneFormSet(BaseInlineFormSet):
    """Require at least one form in the formset to be completed.
    http://wadofstuff.blogspot.it/2009/08/requiring-at-least-one-inline-formset.html"
    
    Use it in your inlineadmin declaring it in the formset variable
    
    class YourInlineAdmin(TabularInline):
        formset=RequireOneFormSet
        extra=...
    """
    
    def clean(self):
        """Check that at least one form has been completed."""
        for error in self.errors:
            if error:
                return
        completed = 0
        for cleaned_data in self.cleaned_data:
            # form has data and we aren't deleting it.
            if cleaned_data and not cleaned_data.get('DELETE', False):
                completed += 1

        if completed < 1:
            raise forms.ValidationError(_("At least one %s is required." %
                unicode(self.model._meta.verbose_name)))
